/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { MenuContainerComponent } from '@admin-client/configuration';
import { ConfigurationLinkRel, ConfigurationResource, ConfigurationService } from '@admin-client/configuration-shared';
import { ROUTES } from '@admin-client/shared';
import { KeycloakTokenService } from '@admin/keycloak-shared';
import { ApiRootLinkRel, ApiRootResource, ApiRootService } from '@alfa-client/api-root-shared';
import { BuildInfoComponent } from '@alfa-client/common';
import { HasLinkPipe, isLoaded, isNotUndefined, mapToResource, StateResource } from '@alfa-client/tech-shared';
import { CommonModule } from '@angular/common';
import { Component, inject, OnInit } from '@angular/core';
import { Router, RouterLink, RouterOutlet } from '@angular/router';
import { AuthenticationService } from '@authentication';
import { hasLink } from '@ngxp/rest';
import { AdminLogoIconComponent, NavbarComponent, NavItemComponent, OrgaUnitIconComponent, UsersIconComponent, } from '@ods/system';
import { filter, Observable, Subscription } from 'rxjs';
import { UserProfileButtonContainerComponent } from '../../../../libs/admin/user-profile/src/lib/user-menu/user-profile.button-container.component';
import { UnavailablePageComponent } from '../pages/unavailable/unavailable-page/unavailable-page.component';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss'],
  standalone: true,
  imports: [
    CommonModule,
    NavbarComponent,
    NavItemComponent,
    UsersIconComponent,
    UserProfileButtonContainerComponent,
    AdminLogoIconComponent,
    OrgaUnitIconComponent,
    RouterOutlet,
    UnavailablePageComponent,
    BuildInfoComponent,
    MenuContainerComponent,
    RouterLink,
    HasLinkPipe,
  ],
})
export class AppComponent implements OnInit {
  readonly title: string = 'admin';

  private readonly authenticationService = inject(AuthenticationService);
  private readonly apiRootService = inject(ApiRootService);
  private readonly router = inject(Router);
  private readonly configurationService = inject(ConfigurationService);
  private readonly keycloakTokenService = inject(KeycloakTokenService);

  public apiRootStateResource$: Observable<StateResource<ApiRootResource>>;

  apiRootSubscription: Subscription;
  configurationSubscription: Subscription;

  public readonly apiRootLinkRel = ApiRootLinkRel;
  public readonly routes = ROUTES;

  ngOnInit(): void {
    this.authenticationService.login().then(() => this._doAfterLoggedIn());
  }

  _doAfterLoggedIn(): void {
    this.apiRootStateResource$ = this.apiRootService.getApiRoot();
    this.keycloakTokenService.registerAccessTokenProvider();
    this._evaluateInitialNavigation();
  }

  _evaluateInitialNavigation(): void {
    if (this.router.url.startsWith('/?state')) {
      this._subscribeApiRootForEvaluation();
    } else {
      this.refreshForGuardEvaluation();
    }
  }

  _subscribeApiRootForEvaluation(): void {
    this.apiRootSubscription = this.apiRootStateResource$
      .pipe(filter(isLoaded), mapToResource<ApiRootResource>())
      .subscribe((apiRootResource: ApiRootResource) => this._evaluateNavigationByApiRoot(apiRootResource));
  }

  private refreshForGuardEvaluation(): void {
    this.router.navigate([window.location.pathname]);
  }

  _evaluateNavigationByApiRoot(apiRootResource: ApiRootResource): void {
    if (hasLink(apiRootResource, ApiRootLinkRel.CONFIGURATION)) {
      this._evaluateNavigationByConfiguration();
    } else {
      this._navigateByApiRoot(apiRootResource);
    }
  }

  _evaluateNavigationByConfiguration(): void {
    this.configurationSubscription = this.configurationService
      .get()
      .pipe(filter(isLoaded), mapToResource<ApiRootResource>())
      .subscribe((configurationResource: ConfigurationResource) => this._navigateByConfiguration(configurationResource));
  }

  _navigateByConfiguration(configurationResource: ConfigurationResource): void {
    if (hasLink(configurationResource, ConfigurationLinkRel.SETTING)) {
      this.navigate(ROUTES.POSTFACH);
    } else if (hasLink(configurationResource, ConfigurationLinkRel.AGGREGATION_MAPPINGS)) {
      this.navigate(ROUTES.AGGREGATION_MAPPING);
    } else {
      this.navigate(ROUTES.UNAVAILABLE);
    }
    this._unsubscribe();
  }

  _navigateByApiRoot(apiRootResource: ApiRootResource): void {
    if (hasLink(apiRootResource, ApiRootLinkRel.USERS)) {
      this.navigate(ROUTES.BENUTZER);
    } else {
      this.navigate(ROUTES.UNAVAILABLE);
    }
    this._unsubscribe();
  }

  private navigate(routePath: string): void {
    this.router.navigate(['/' + routePath]);
  }

  _unsubscribe(): void {
    if (isNotUndefined(this.apiRootSubscription)) this.apiRootSubscription.unsubscribe();
    if (isNotUndefined(this.configurationSubscription)) this.configurationSubscription.unsubscribe();
  }
}
