/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  createPostfachNachrichtAttachedItem,
  createPostfachNachrichtReplyItem,
} from 'apps/alfa-e2e/src/support/postfach-nachricht.util';
import { PostfachMailE2EComponent, PostfachMailListItem } from '../../../components/postfach/postfach-mail.e2e.component';
import { SnackBarE2EComponent } from '../../../components/ui/snackbar.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangSubnavigationE2EComponent } from '../../../components/vorgang/vorgang-subnavigation';
import { VorgangE2E } from '../../../model/vorgang';
import {
  DirectionE2E,
  PostfachMailItemE2E,
  PostfachNachrichtMessageCodeE2E,
  PostfachNachrichtMessageCodeMessagesE2E,
  PostfachNachrichtSnackbarMessageE2E,
  VorgangAttachedItemE2E,
} from '../../../model/vorgang-attached-item';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { PostfachMailPage } from '../../../page-objects/postfach-mail.component.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections } from '../../../support/cypress-helper';
import { contains, exist, notExist } from '../../../support/cypress.util';
import { getUserSabineId, loginAsSabine } from '../../../support/user-util';
import { initVorgangAttachedItem } from '../../../support/vorgang-attached-item-util';
import { createVorgang, initVorgang, objectIds } from '../../../support/vorgang-util';

describe('PostfachMail error', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
  const snackbar: SnackBarE2EComponent = mainPage.getSnackBar();

  const vorgangPage: VorgangPage = new VorgangPage();
  const postfachMailContainer: PostfachMailE2EComponent = vorgangPage.getPostfachMailcontainer();
  const subnavigation: VorgangSubnavigationE2EComponent = vorgangPage.getSubnavigation();

  const postfachMailPage: PostfachMailPage = new PostfachMailPage();

  const vorgang: VorgangE2E = createVorgang();
  const postfachMailItem: PostfachMailItemE2E = {
    ...createPostfachNachrichtReplyItem(),
    createdBy: getUserSabineId(),
    direction: DirectionE2E.OUT,
    sentAt: '2022-12-02T15:00:00.790Z[UTC]',
    sentSuccessful: false,
    messageCode: PostfachNachrichtMessageCodeE2E.PROCESSING_FAILED,
  };

  const vorgangAttachedItem: VorgangAttachedItemE2E = {
    ...createPostfachNachrichtAttachedItem(objectIds[0], vorgang._id.$oid),
    item: postfachMailItem,
  };

  before(() => {
    initVorgang(vorgang);
    initVorgangAttachedItem([vorgangAttachedItem]);

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('navigate to vorgang detail', () => {
    it('should open vorgang detail', () => {
      vorgangList.getListItem(vorgang.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    it('should show postfach mail button', () => {
      exist(postfachMailContainer.getCreateButtonWithText());
    });

    it('should show postfach mail button in subnavigation', () => {
      exist(subnavigation.getPostfachMailIconButton());
    });

    it('should show postfach mail list', () => {
      exist(postfachMailContainer.getList());
    });
  });

  describe('postfach mail list', () => {
    let listItem: PostfachMailListItem;

    beforeEach(() => {
      listItem = postfachMailPage.getListItem(postfachMailItem.subject);
    });

    describe('contains failed sent mail item', () => {
      it('should show mail subject', () => {
        contains(listItem.getSubject(), postfachMailItem.subject);
      });

      it('should show user profile', () => {
        exist(listItem.getUserProfile().getRoot());
      });

      it('should show sent date', () => {
        exist(listItem.getSentAt());
      });

      it('should show text', () => {
        contains(listItem.getMailSendErrorText(), PostfachNachrichtMessageCodeMessagesE2E.PROCESSING_FAILED);
      });

      it('should show error message', () => {
        exist(listItem.getMailSendErrorIcon());
      });

      it('should show button', () => {
        exist(listItem.getResendButton());
      });
    });

    describe('resend mail', () => {
      it('click on resend button should hide error', () => {
        listItem.getResendButton().click();
        waitForSpinnerToDisappear();

        notExist(listItem.getMailSendErrorIcon());
      });

      describe('resend mail item', () => {
        it('should show snackbar', () => {
          contains(snackbar.getMessage(), PostfachNachrichtSnackbarMessageE2E.SEND_SUCCESSFUL);
          notExist(snackbar.getRevokeButton());
        });

        it('should close snackbar on close button', () => {
          snackbar.getCloseButton().click();

          notExist(snackbar.getMessage());
        });

        it('should show mail subject', () => {
          contains(listItem.getSubject(), postfachMailItem.subject);
        });

        it('should show user profile', () => {
          exist(listItem.getUserProfile().getRoot());
        });

        it('should show date', () => {
          exist(listItem.getSentAt());
        });

        it('should show text', () => {
          contains(listItem.getText(), postfachMailItem.mailBody);
        });
      });
    });
  });
});
