/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { VorgangFormularButtonsE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-formular-buttons.e2e.components';
import { APP_HELPER } from 'apps/alfa-e2e/src/helper/app.const';
import { VORGANG_EXECUTOR, VORGANG_NAVIGATOR, VORGANG_VERIFIER } from 'apps/alfa-e2e/src/helper/vorgang/vorgang.const';
import { SnackBarE2EComponent } from '../../../components/ui/snackbar.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangMessagesE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections } from '../../../support/cypress-helper';
import { contains, exist, haveText, notExist } from '../../../support/cypress.util';
import { loginAsSabine } from '../../../support/user-util';
import { buildVorgang, createVorgang, initVorgaenge, objectIds } from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang Zurueckholen', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
  const snackBar: SnackBarE2EComponent = mainPage.getSnackBar();

  const vorgangPage: VorgangPage = new VorgangPage();
  const vorgangFormularButtons: VorgangFormularButtonsE2EComponent = vorgangPage.getFormularButtons();

  const vorgangZurueckholen: VorgangE2E = {
    ...createVorgang(),
    name: 'DoZurueckholen',
    status: VorgangStatusE2E.VERWORFEN,
  };
  const vorgangZurueckholenRevoke: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'DoRevokeZurueckholen'),
    status: VorgangStatusE2E.VERWORFEN,
  };

  before(() => {
    initVorgaenge([vorgangZurueckholen, vorgangZurueckholenRevoke]);

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('Zurueckholen', () => {
    it('Open Vorgang-Detail-Page', () => {
      vorgangList.getListItem(vorgangZurueckholen.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    describe('by button', () => {
      it('should have status Verworfen', () => {
        haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangZurueckholen.status]);
      });

      it('should show snackBar message', () => {
        vorgangFormularButtons.getZurueckholenButton().click();
        waitForSpinnerToDisappear();

        exist(snackBar.getCloseButton());
        contains(snackBar.getMessage(), VorgangMessagesE2E.ZURUECKGEHOLT);
      });

      it('should close snackBar on close button', () => {
        snackBar.getCloseButton().click();

        notExist(snackBar.getMessage());
      });

      it('should have status Neu', () => {
        haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.NEU]);
      });

      it('should have button annehmen und verwerfen', () => {
        exist(vorgangFormularButtons.getVerwerfenButton());
      });

      it('back to vorgang list', () => {
        vorgangPage.getSubnavigation().getBackButton().click();
        waitForSpinnerToDisappear();

        exist(vorgangList.getRoot());
      });

      it('should have status Neu', () => {
        haveText(vorgangList.getListItem(vorgangZurueckholen.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.NEU]);
      });
    });
  });

  describe('Zurueckholen und rueckgaengig machen', () => {
    describe('by icon-button', () => {
      it('should show vorgang status', () => {
        VORGANG_NAVIGATOR.openVorgang(vorgangZurueckholenRevoke.name);

        VORGANG_VERIFIER.verifyStatus(VorgangStatusE2E.VERWORFEN);
      });

      describe('by icon-button', () => {
        it('should have status Neu', () => {
          VORGANG_EXECUTOR.zurueckholenByIconButton();

          APP_HELPER.revokeAndCloseSnackbar(VorgangMessagesE2E.ZURUECKGEHOLT);

          VORGANG_VERIFIER.verifyStatus(VorgangStatusE2E.VERWORFEN);
        });

        it('should have reverted status in list after navigate back', () => {
          VORGANG_NAVIGATOR.navigateBack();

          VORGANG_VERIFIER.verifyStatusInList(vorgangZurueckholenRevoke.name, VorgangStatusE2E.VERWORFEN);
        });
      });
    });
  });
});
