/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { NavigationE2EComponent } from 'apps/alfa-e2e/src/components/navigation/navigation.e2e.component';
import { VorgangListE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-list.e2e.component';
import {
  VorgangViewE2EComponent,
  VorgangViewsE2EComponent,
} from 'apps/alfa-e2e/src/components/vorgang/vorgang-views.e2e.component';
import { MainPage, waitForSpinnerToDisappear } from 'apps/alfa-e2e/src/page-objects/main.po';
import { exist } from 'apps/alfa-e2e/src/support/cypress.util';
import { Interception } from 'cypress/types/net-stubbing';
import { HttpMethodE2E } from '../../../model/util';
import { VorgangE2E, VorgangStatusE2E } from '../../../model/vorgang';
import {
  dropCollections,
  intercept,
  scrollToWindowBottom,
  scrollToWindowTop,
  waitOfInterceptor,
} from '../../../support/cypress-helper';
import { generateIds } from '../../../support/tech.util';
import { getUserSabineId, initUsermanagerUsers, loginAsSabine } from '../../../support/user-util';
import { buildVorgang, initVorgaenge } from '../../../support/vorgang-util';

describe('VorgangList View Pages', () => {
  const mainPage: MainPage = new MainPage();

  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const navigation: NavigationE2EComponent = mainPage.getNavigation();

  const views: VorgangViewsE2EComponent = mainPage.getViews();

  const VORGANG_NAME_PREFIX: string = 'Vorgang';
  const PAGE_SIZE: number = 100;
  const assignedTo: string = getUserSabineId();

  before(() => {
    const vorgaenge: VorgangE2E[] = createVorgaengeWithStatus(VorgangStatusE2E.NEU).concat(
      createAssignedVorgaengeWithStatus(VorgangStatusE2E.NEU),
      createVorgaengeWithStatus(VorgangStatusE2E.ANGENOMMEN),
      createAssignedVorgaengeWithStatus(VorgangStatusE2E.ANGENOMMEN),
      createVorgaengeWithStatus(VorgangStatusE2E.IN_BEARBEITUNG),
      createAssignedVorgaengeWithStatus(VorgangStatusE2E.IN_BEARBEITUNG),
      createVorgaengeWithStatus(VorgangStatusE2E.BESCHIEDEN),
      createAssignedVorgaengeWithStatus(VorgangStatusE2E.BESCHIEDEN),
      createVorgaengeWithStatus(VorgangStatusE2E.ABGESCHLOSSEN),
      createAssignedVorgaengeWithStatus(VorgangStatusE2E.ABGESCHLOSSEN),
      createVorgaengeWithStatus(VorgangStatusE2E.WEITERGELEITET),
      createAssignedVorgaengeWithStatus(VorgangStatusE2E.WEITERGELEITET),
      createVorgaengeWithStatus(VorgangStatusE2E.VERWORFEN),
      createAssignedVorgaengeWithStatus(VorgangStatusE2E.VERWORFEN),
    );
    initVorgaenge(vorgaenge);
    initUsermanagerUsers();

    loginAsSabine();
    exist(vorgangList.getRoot());
    waitForSpinnerToDisappear();
  });

  after(() => {
    dropCollections();
  });

  describe('Neu', () => {
    const STATUS: string = VorgangStatusE2E.NEU;

    testAlleFilter(STATUS, views.getNeu());
    testMeineVorgaengeFilter(STATUS);
    testNichtZugewiesenFilter(STATUS);
  });

  describe('Angenommen', () => {
    const STATUS: string = VorgangStatusE2E.ANGENOMMEN;

    testAlleFilter(STATUS, views.getAngenommen());
    testMeineVorgaengeFilter(STATUS);
    testNichtZugewiesenFilter(STATUS);
  });

  describe('In Bearbeitung', () => {
    const STATUS: string = VorgangStatusE2E.IN_BEARBEITUNG;

    testAlleFilter(STATUS, views.getInBearbeitung());
    testMeineVorgaengeFilter(STATUS);
    testNichtZugewiesenFilter(STATUS);
  });

  describe('Bescheiden', () => {
    const STATUS: string = VorgangStatusE2E.BESCHIEDEN;

    testAlleFilter(STATUS, views.getBeschieden());
    testMeineVorgaengeFilter(STATUS);
    testNichtZugewiesenFilter(STATUS);
  });

  describe('Abgeschlossen', () => {
    const STATUS: string = VorgangStatusE2E.ABGESCHLOSSEN;

    testAlleFilter(STATUS, views.getAbgeschlossen());
    testMeineVorgaengeFilter(STATUS);
    testNichtZugewiesenFilter(STATUS);
  });

  describe('Weitergeleitet', () => {
    const STATUS: string = VorgangStatusE2E.WEITERGELEITET;

    testAlleFilter(STATUS, views.getWeitergeleitet());
    testMeineVorgaengeFilter(STATUS);
    testNichtZugewiesenFilter(STATUS);
  });

  describe('Verworfen', () => {
    const STATUS: string = VorgangStatusE2E.VERWORFEN;

    testAlleFilter(STATUS, views.getVerworfen());
    testMeineVorgaengeFilter(STATUS);
    testNichtZugewiesenFilter(STATUS);
  });

  function testAlleFilter(status: string, view: VorgangViewE2EComponent) {
    describe('Alle filter', () => {
      testAlleFilterPage1(status, view);
      testAlleFilterPage2(status);
      testAlleFilterPage3(status);
    });
  }

  function testAlleFilterPage1(status: string, view: VorgangViewE2EComponent): void {
    describe('page 1', () => {
      const loadPageInterceptor: string = 'loadPage';

      beforeEach(() => {
        navigation.getAlleFilter().click();
        waitForSpinnerToDisappear();
        intercept(HttpMethodE2E.GET, buildVorgangStatusPageLink(1, status)).as(loadPageInterceptor);
      });

      it(`should show page 1 with 100 vorgaenge ${status}`, () => {
        view.getRoot().click();

        waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
          assert(getVorgangList(interception).length === PAGE_SIZE);
          assert(getVorgangList(interception)[0].name, VORGANG_NAME_PREFIX + status + 1);
          assert(getVorgangList(interception)[99].name, VORGANG_NAME_PREFIX + status + PAGE_SIZE);
        });
      });
    });
  }

  function testAlleFilterPage2(status: string): void {
    describe('page 2', () => {
      const loadPageInterceptor: string = 'loadPage';

      beforeEach(() => {
        intercept(HttpMethodE2E.GET, buildVorgangStatusPageLink(2, status)).as(loadPageInterceptor);
      });

      it(`should show page 2 with 10 vorgaenge ${status} and 90 vorgaenge ASSIGNED on scrolling to bottom`, () => {
        scrollToWindowBottom();

        waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
          assert(getVorgangList(interception).length === PAGE_SIZE);
          assert(getVorgangList(interception)[0].name, VORGANG_NAME_PREFIX + status + 'Assigned_' + 1);
          assert(getVorgangList(interception)[99].name, VORGANG_NAME_PREFIX + status + 'Assigned_' + PAGE_SIZE);
        });
      });
    });
  }

  function testAlleFilterPage3(status: string): void {
    describe('page 3', () => {
      const loadPageInterceptor: string = 'loadPage';

      beforeEach(() => {
        intercept(HttpMethodE2E.GET, buildVorgangStatusPageLink(3, status)).as(loadPageInterceptor);
      });

      after(() => {
        scrollToWindowTop();
      });

      it(`should show page 3 with 10 vorgaenge ${status} ASSIGNED on scrolling to bottom`, () => {
        scrollToWindowBottom();

        waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
          assert(getVorgangList(interception).length === 20);
          assert(getVorgangList(interception)[0].name, VORGANG_NAME_PREFIX + status + 'Assigned_' + 80);
          assert(getVorgangList(interception)[19].name, VORGANG_NAME_PREFIX + status + 'Assigned_' + PAGE_SIZE);
        });
      });
    });
  }

  function buildVorgangStatusPageLink(page: number, status: string): string {
    return `${buildVorgangPageLink(page)}&status=${status}`;
  }

  function testMeineVorgaengeFilter(status: string): void {
    describe('Meine Vorgänge filter', () => {
      testMeineVorgaengePage1(status);
      testMeineVorgaengePage2(status);
    });
  }

  function testMeineVorgaengePage1(status: string): void {
    describe('page 1', () => {
      const loadPageInterceptor: string = 'loadPage';

      beforeEach(() => {
        intercept(HttpMethodE2E.GET, buildVorgangInStatusAssignedPageLink(1, status)).as(loadPageInterceptor);
      });

      it(`should show page 1 with 100 vorgaenge ${status}`, () => {
        navigation.getMeineVorgaengeFilter().click();

        waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
          assert(getVorgangList(interception).length === PAGE_SIZE);
          assert(getVorgangList(interception)[0].name, VORGANG_NAME_PREFIX + 'Assigned_' + 1);
          assert(getVorgangList(interception)[99].name, VORGANG_NAME_PREFIX + 'Assigned_' + PAGE_SIZE);
        });
      });
    });
  }

  function testMeineVorgaengePage2(status: string): void {
    describe('page 2', () => {
      const loadPageInterceptor: string = 'loadPage';

      beforeEach(() => {
        intercept(HttpMethodE2E.GET, buildVorgangInStatusAssignedPageLink(2, status)).as(loadPageInterceptor);
      });

      after(() => {
        scrollToWindowTop();
      });

      it(`should show page 2 with 10 vorgaenge ${status} and 90 vorgaenge ASSIGNED on scrolling to bottom`, () => {
        scrollToWindowBottom();

        waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
          assert(getVorgangList(interception).length === 10);
          assert(getVorgangList(interception)[0].name, VORGANG_NAME_PREFIX + 'Assigned_' + 101);
          assert(getVorgangList(interception)[9].name, VORGANG_NAME_PREFIX + 'Assigned_' + 110);
        });
      });
    });
  }

  function buildVorgangInStatusAssignedPageLink(page: number, status: string): string {
    return `${buildVorgangPageLink(page)}&assignedTo=${assignedTo}&status=${status}`;
  }

  function testNichtZugewiesenFilter(status: string): void {
    describe('Nicht zugewiesen filter', () => {
      testNichtZugewiesenPage1(status);
      testNichtZugewiesenPage2(status);
    });
  }

  function testNichtZugewiesenPage1(status: string): void {
    describe('page 1', () => {
      const loadPageInterceptor: string = 'loadPage';

      beforeEach(() => {
        intercept(HttpMethodE2E.GET, buildVorgangInStatusUnassignedPageLink(1, status)).as(loadPageInterceptor);
      });

      it(`should show page 1 with 100 vorgaeng ${status}`, () => {
        navigation.getUnassignedFilter().click();

        waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
          assert(getVorgangList(interception).length === PAGE_SIZE);
          assert(getVorgangList(interception)[0].name, VORGANG_NAME_PREFIX + status + 'Assigned_' + 1);
          assert(getVorgangList(interception)[99].name, VORGANG_NAME_PREFIX + status + 'Assigned_' + PAGE_SIZE);
        });
      });
    });
  }

  function testNichtZugewiesenPage2(status: string): void {
    describe('page 2', () => {
      const loadPageInterceptor: string = 'loadPage';

      beforeEach(() => {
        intercept(HttpMethodE2E.GET, buildVorgangInStatusUnassignedPageLink(2, status)).as(loadPageInterceptor);
      });

      it('should show page 2 with 10 vorgaenge on scrolling to bottom', () => {
        scrollToWindowBottom();

        waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
          assert(getVorgangList(interception).length === 10);
          assert(getVorgangList(interception)[0].name, VORGANG_NAME_PREFIX + status + 'Assigned_' + 101);
          assert(getVorgangList(interception)[9].name, VORGANG_NAME_PREFIX + status + 'Assigned_' + 110);
        });
        scrollToWindowTop();
      });
    });
  }

  function buildVorgangInStatusUnassignedPageLink(page: number, status: string): string {
    return `${buildVorgangPageLink(page)}&assignedTo=&status=${status}`;
  }

  function buildVorgangPageLink(page: number): string {
    return `vorgangs?page=${page - 1}&limit=100`;
  }

  function getVorgangList(interception: Interception): VorgangE2E[] {
    return interception.response.body._embedded.vorgangHeaderList;
  }

  function createVorgaengeWithStatus(status: VorgangStatusE2E): VorgangE2E[] {
    let count = 1;
    return generateIds(110).map((randomId) => {
      const vorgang: VorgangE2E = {
        ...buildVorgang(randomId, VORGANG_NAME_PREFIX + status + '_' + count),
        status,
      };
      count++;
      return vorgang;
    });
  }

  function createAssignedVorgaengeWithStatus(status: VorgangStatusE2E): VorgangE2E[] {
    let count = 1;
    return generateIds(110).map((randomId) => {
      const vorgang: VorgangE2E = {
        ...buildVorgang(randomId, VORGANG_NAME_PREFIX + status + 'Assigned_' + count),
        status,
        assignedTo,
      };
      count++;
      return vorgang;
    });
  }
});
