import { ComponentFixture, TestBed } from '@angular/core/testing';

import { AdminOrganisationsEinheitService } from '@admin-client/organisations-einheit-shared';
import { createEmptyStateResource, StateResource } from '@alfa-client/tech-shared';
import { dispatchEventFromFixture, getMockComponent, Mock, mock } from '@alfa-client/test-utils';
import { OzgcloudDialogService } from '@alfa-client/ui';
import { DIALOG_DATA } from '@angular/cdk/dialog';
import { faker } from '@faker-js/faker';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { singleColdCompleted } from 'libs/tech-shared/test/marbles';
import { Observable, of } from 'rxjs';
import { OrganisationsEinheitDeleteDialogContainerComponent } from './organisations-einheit-delete-dialog-container.component';
import { OrganisationsEinheitDeleteDialogComponent } from './organisations-einheit-delete-dialog/organisations-einheit-delete-dialog.component';

describe('OrganisationsEinheitDeleteDialogContainerComponent', () => {
  let component: OrganisationsEinheitDeleteDialogContainerComponent;
  let fixture: ComponentFixture<OrganisationsEinheitDeleteDialogContainerComponent>;

  let dialogService: Mock<OzgcloudDialogService>;
  let organisationsEinheitService: Mock<AdminOrganisationsEinheitService>;

  const dialogData = { organisationsEinheitName: faker.word.sample(), organisationsEinheitId: faker.string.uuid() };

  const organisationsEinheitDeleteDialog: string = getDataTestIdOf('organisations-einheit-delete-dialog');

  beforeEach(() => {
    dialogService = mock(OzgcloudDialogService);
    organisationsEinheitService = { ...mock(AdminOrganisationsEinheitService), delete: jest.fn() };

    TestBed.configureTestingModule({
      imports: [OrganisationsEinheitDeleteDialogContainerComponent],
      providers: [
        { provide: OzgcloudDialogService, useValue: dialogService },
        { provide: AdminOrganisationsEinheitService, useValue: organisationsEinheitService },
        {
          provide: DIALOG_DATA,
          useValue: dialogData,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(OrganisationsEinheitDeleteDialogContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('template', () => {
    describe('organisations einheit delete dialog', () => {
      it('should exist', () => {
        const stateResource: StateResource<void> = createEmptyStateResource();
        component.deleteStateResource$ = of(stateResource);
        component.organisationsEinheitName = dialogData.organisationsEinheitName;

        const deleteDialogComponent: OrganisationsEinheitDeleteDialogComponent = getMockComponent(
          fixture,
          OrganisationsEinheitDeleteDialogComponent,
        );

        expect(deleteDialogComponent.organisationsEinheitName).toBe(component.organisationsEinheitName);
        expect(deleteDialogComponent.deleteStateResource).toEqual(stateResource);
      });

      it('should call closeDialog on cancel emit', () => {
        component.closeDialog = jest.fn();

        dispatchEventFromFixture(fixture, organisationsEinheitDeleteDialog, 'cancel');

        expect(component.closeDialog).toHaveBeenCalled();
      });

      it('should call delete on delete emit', () => {
        component.delete = jest.fn();

        dispatchEventFromFixture(fixture, organisationsEinheitDeleteDialog, 'delete');

        expect(component.delete).toHaveBeenCalled();
      });
    });
  });

  describe('component', () => {
    describe('constructor', () => {
      it('should set organisationsEinheitName', () => {
        expect(component.organisationsEinheitName).toBe(dialogData.organisationsEinheitName);
      });

      it('should set organisationsEinheitId', () => {
        expect(component.organisationsEinheitId).toBe(dialogData.organisationsEinheitId);
      });
    });

    describe('closeDialog', () => {
      it('should call dialogService closeAll', () => {
        dialogService.closeAll = jest.fn();

        component.closeDialog();

        expect(dialogService.closeAll).toHaveBeenCalled();
      });
    });

    describe('delete', () => {
      const stateResource: StateResource<void> = createEmptyStateResource();
      const stateResource$: Observable<StateResource<void>> = of(stateResource);
      const loadingStateResource: StateResource<void> = createEmptyStateResource(true);
      const loadingStateResource$: Observable<StateResource<void>> = of(loadingStateResource);

      beforeEach(() => {
        organisationsEinheitService.delete.mockReturnValue(stateResource$);
      });

      it('should call organisationsEinheitService delete', () => {
        component.delete();

        expect(organisationsEinheitService.delete).toHaveBeenCalledWith(dialogData.organisationsEinheitId);
      });

      it('should set deleteStateResource$', () => {
        component.delete();

        expect(component.deleteStateResource$).toBeObservable(singleColdCompleted(stateResource));
      });

      it('should close dialog on delete done', () => {
        component.delete();
        component.deleteStateResource$.subscribe();

        expect(dialogService.closeAll).toHaveBeenCalled();
      });

      it('should NOT close dialog on delete loading', () => {
        organisationsEinheitService.delete.mockReturnValue(loadingStateResource$);

        component.delete();
        component.deleteStateResource$.subscribe();

        expect(dialogService.closeAll).not.toHaveBeenCalled();
      });
    });
  });
});
