import { BescheidDocument, createEmptyBescheidDocument, createEmptyUploadInProgress } from '@alfa-client/bescheid-shared';
import { BinaryFile2ContainerComponent } from '@alfa-client/binary-file';
import {
  ConvertProblemDetailToErrorMessagesPipe,
  createEmptyStateResource,
  createErrorStateResource,
  createLoadingStateResource,
  ProblemDetail,
} from '@alfa-client/tech-shared';
import {
  existsAsHtmlElement,
  getElementComponentFromFixtureByCss,
  notExistsAsHtmlElement,
  triggerEvent,
} from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { AttachmentComponent, AttachmentWrapperComponent } from '@ods/system';
import { MockComponent } from 'ng-mocks';
import {
  createBescheidDocument,
  createErrorUpload,
  createPendingUpload,
  createSuccessfulUpload,
} from '../../../../../../../bescheid-shared/src/test/bescheid';
import { createSuccessfullyDoneCommandStateResource } from '../../../../../../../command-shared/test/command';
import { getDataTestIdOf } from '../../../../../../../tech-shared/test/data-test';
import { createProblemDetail } from '../../../../../../../tech-shared/test/error';
import { BescheidWizardDocumentFileComponent } from './bescheid-wizard-document-file.component';

describe('BescheidWizardDocumentFileComponent', () => {
  let component: BescheidWizardDocumentFileComponent;
  let fixture: ComponentFixture<BescheidWizardDocumentFileComponent>;

  const convertProblemDetailsToErrorMessages: ConvertProblemDetailToErrorMessagesPipe =
    new ConvertProblemDetailToErrorMessagesPipe();

  const createBescheidTestId: string = getDataTestIdOf('create-bescheid-document-attachment');
  const uploadBescheidTestId: string = getDataTestIdOf('upload-bescheid-document');
  const bescheidDocumentTestId: string = getDataTestIdOf('bescheid-document-binary-file');

  const bescheidDocument: BescheidDocument = createBescheidDocument();

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BescheidWizardDocumentFileComponent,
        MockComponent(AttachmentWrapperComponent),
        MockComponent(AttachmentComponent),
        MockComponent(BinaryFile2ContainerComponent),
        ConvertProblemDetailToErrorMessagesPipe,
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardDocumentFileComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  describe('component', () => {
    it('should create', () => {
      expect(component).toBeTruthy();
    });

    it('should set initial values', () => {
      expect(component.showAutomaticCreation).toEqual(false);
      expect(component.showUpload).toEqual(false);
      expect(component.showDocument).toEqual(false);
      expect(component.createDocumentErrorMessages).toEqual([]);
      expect(component.bescheidDocument).toEqual(createEmptyBescheidDocument());
    });

    describe('set bescheid document', () => {
      it('should update component state', () => {
        component.update = jest.fn();

        component.bescheidDocument = bescheidDocument;

        expect(component.update).toHaveBeenCalled();
      });
    });

    describe('update', () => {
      it('should set bescheid document', () => {
        component.update(bescheidDocument);

        expect(component.bescheidDocument).toEqual(bescheidDocument);
      });

      it('should show automatic creation on loading', () => {
        component.update({ ...bescheidDocument, create: createLoadingStateResource() });

        expect(component.showAutomaticCreation).toEqual(true);
      });

      it('should show automatic creation on error', () => {
        component.update({
          ...bescheidDocument,
          create: createErrorStateResource(createProblemDetail()),
        });

        expect(component.showAutomaticCreation).toEqual(true);
      });

      it('should show upload on loading', () => {
        component.update({ ...bescheidDocument, upload: createPendingUpload() });

        expect(component.showUpload).toEqual(true);
      });

      it('should show upload on error', () => {
        component.update({
          ...bescheidDocument,
          upload: createErrorUpload(),
        });

        expect(component.showUpload).toEqual(true);
      });

      it('should show created document', () => {
        component.update({
          ...bescheidDocument,
          create: createSuccessfullyDoneCommandStateResource(),
          upload: createEmptyUploadInProgress(),
        });

        expect(component.showDocument).toEqual(true);
      });

      it('should show uploaded document', () => {
        component.update({
          ...bescheidDocument,
          create: createEmptyStateResource(),
          upload: createSuccessfulUpload(),
        });

        expect(component.showDocument).toEqual(true);
      });

      it('should NOT show document on pending creation', () => {
        component.update({
          ...bescheidDocument,
          create: createLoadingStateResource(),
          upload: createEmptyUploadInProgress(),
        });

        expect(component.showDocument).toEqual(false);
      });

      it('should NOT show document on pending upload', () => {
        component.update({
          ...bescheidDocument,
          create: createEmptyStateResource(),
          upload: createPendingUpload(),
        });

        expect(component.showDocument).toEqual(false);
      });

      it('should NOT show document if not exists', () => {
        component.update({
          ...bescheidDocument,
          resource: null,
          create: createEmptyStateResource(),
          upload: createPendingUpload(),
        });

        expect(component.showDocument).toEqual(false);
      });

      it('should have error messages', () => {
        component.update({
          ...bescheidDocument,
          create: createErrorStateResource(createProblemDetail()),
          upload: createEmptyUploadInProgress(),
        });

        expect(component.createDocumentErrorMessages).toEqual([BescheidWizardDocumentFileComponent.CREATE_ERROR_MESSAGE]);
      });

      it('should NOT have error messages', () => {
        component.update({
          ...bescheidDocument,
          create: createSuccessfullyDoneCommandStateResource(),
          upload: createEmptyUploadInProgress(),
        });

        expect(component.createDocumentErrorMessages).toEqual([]);
      });
    });
  });

  describe('template', () => {
    describe('automatic creation attachment', () => {
      it('should exists', () => {
        component.showAutomaticCreation = true;

        fixture.detectChanges();

        existsAsHtmlElement(fixture, createBescheidTestId);
      });

      it('should NOT exists', () => {
        component.showAutomaticCreation = false;

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, createBescheidTestId);
      });

      it('should have been called with inputs', () => {
        component.showAutomaticCreation = true;

        fixture.detectChanges();
        const attachmentComponent: AttachmentComponent = getElementComponentFromFixtureByCss<AttachmentComponent>(
          fixture,
          createBescheidTestId,
        );

        expect(attachmentComponent.isLoading).toEqual(component.bescheidDocument.create.loading);
        expect(attachmentComponent.errorMessages).toEqual(component.createDocumentErrorMessages);
      });
    });

    describe('upload file', () => {
      it('should exists', () => {
        component.showUpload = true;

        fixture.detectChanges();

        existsAsHtmlElement(fixture, uploadBescheidTestId);
      });

      it('should NOT exists', () => {
        component.showUpload = false;

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, uploadBescheidTestId);
      });

      it('should have been called with inputs', () => {
        component.showUpload = true;

        fixture.detectChanges();
        const attachmentComponent: AttachmentComponent = getElementComponentFromFixtureByCss<AttachmentComponent>(
          fixture,
          uploadBescheidTestId,
        );

        expect(attachmentComponent.loadingCaption).toEqual(component.bescheidDocument.upload.fileName);
        expect(attachmentComponent.isLoading).toEqual(component.bescheidDocument.upload.loading);
        expect(attachmentComponent.errorMessages).toEqual(
          convertProblemDetailsToErrorMessages.transform(component.bescheidDocument.upload.error as ProblemDetail),
        );
      });
    });

    describe('document binary file', () => {
      it('should exists', () => {
        component.showDocument = true;

        fixture.detectChanges();

        existsAsHtmlElement(fixture, bescheidDocumentTestId);
      });

      it('should NOT exists', () => {
        component.showDocument = false;

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, bescheidDocumentTestId);
      });

      it('should have been called with inputs', () => {
        component.showDocument = true;

        fixture.detectChanges();
        const attachmentComponent: BinaryFile2ContainerComponent =
          getElementComponentFromFixtureByCss<BinaryFile2ContainerComponent>(fixture, bescheidDocumentTestId);

        expect(attachmentComponent.file).toEqual(component.bescheidDocument.resource);
        expect(attachmentComponent.deletable).toEqual(component.deletable);
      });

      describe('output', () => {
        describe('startDelete', () => {
          it('should emit', () => {
            component.showDocument = true;
            fixture.detectChanges();
            component.deleteFile.emit = jest.fn();

            triggerEvent({ fixture, elementSelector: bescheidDocumentTestId, name: 'startDelete' });

            expect(component.deleteFile.emit).toHaveBeenCalled();
          });
        });
      });
    });
  });
});
