/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  CommandProps,
  CommandResource,
  CreateCommandFailureProps,
  CreateCommandProps,
} from '@alfa-client/command-shared';
import {
  ApiError,
  ApiErrorAction,
  StateResource,
  createEmptyStateResource,
  createErrorStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import { Action, ActionReducer, createReducer, on } from '@ngrx/store';
import { LoeschAnforderungResource } from '../loesch-anforderung.model';
import {
  isLoeschAnforderungZuruecknehmen,
  isVorgangLoeschenCommand,
  isVorgangZumLoeschenMarkierenCommand,
} from '../loesch-anforderung.util';
import { LoeschAnforderungProps } from './loesch-anforderung.actions';

import * as CommandActions from '../../../../command-shared/src/lib/+state/command.actions';
import * as NavigationActions from '../../../../navigation-shared/src/lib/+state/navigation.actions';
import * as VorgangActions from '../../../../vorgang-shared/src/lib/+state/vorgang.actions';
import * as LoeschAnforderungActions from './loesch-anforderung.actions';

export const LOESCH_ANFORDERUNG_FEATURE_KEY = 'LoeschAnforderungState';

export interface LoeschAnforderungPartialState {
  readonly [LOESCH_ANFORDERUNG_FEATURE_KEY]: LoeschAnforderungState;
}

export interface LoeschAnforderungState {
  loeschenAnfordernCommand: StateResource<CommandResource>;
  loeschAnforderung: StateResource<LoeschAnforderungResource>;
  vorgangLoeschenCommand: StateResource<CommandResource>;
  loeschAnforderungZuruecknehmenCommand: StateResource<CommandResource>;
}

export const initialState: LoeschAnforderungState = {
  loeschenAnfordernCommand: createEmptyStateResource(),
  loeschAnforderung: createEmptyStateResource(),
  vorgangLoeschenCommand: createEmptyStateResource(),
  loeschAnforderungZuruecknehmenCommand: createEmptyStateResource(),
};

const loeschAnforderungReducer: ActionReducer<LoeschAnforderungState, Action> = createReducer(
  initialState,

  on(
    LoeschAnforderungActions.loadLoeschAnforderungSuccess,
    (state: LoeschAnforderungState, props: LoeschAnforderungProps): LoeschAnforderungState => ({
      ...state,
      loeschAnforderung: createStateResource(props.loeschAnforderung),
    }),
  ),
  on(
    LoeschAnforderungActions.loadLoeschAnforderungFailure,
    (state: LoeschAnforderungState, props: ApiErrorAction): LoeschAnforderungState => ({
      ...state,
      loeschAnforderung: createErrorStateResource(props.apiError),
    }),
  ),

  //LoeschenAnfordernCommand
  on(
    CommandActions.createCommand,
    (state: LoeschAnforderungState, props: CreateCommandProps): LoeschAnforderungState => {
      return isVorgangZumLoeschenMarkierenCommand(props.command.order) ?
          {
            ...state,
            loeschenAnfordernCommand: createEmptyStateResource(true),
          }
        : state;
    },
  ),
  on(
    CommandActions.createCommandSuccess,
    (state: LoeschAnforderungState, props: CommandProps): LoeschAnforderungState => {
      return isVorgangZumLoeschenMarkierenCommand(props.command.order) ?
          {
            ...state,
            loeschenAnfordernCommand: createStateResource(props.command),
          }
        : state;
    },
  ),
  on(
    CommandActions.revokeCommandSuccess,
    (state: LoeschAnforderungState, props: CommandProps): LoeschAnforderungState => {
      return isVorgangZumLoeschenMarkierenCommand(props.command.order) ?
          {
            ...state,
            loeschAnforderung: createEmptyStateResource(),
          }
        : state;
    },
  ),

  //VorgangLoeschenCommand
  on(
    CommandActions.createCommand,
    (state: LoeschAnforderungState, props: CreateCommandProps): LoeschAnforderungState => {
      return isVorgangLoeschenCommand(props.command.order) ?
          { ...state, vorgangLoeschenCommand: createEmptyStateResource(true) }
        : state;
    },
  ),
  on(
    CommandActions.createCommandSuccess,
    (state: LoeschAnforderungState, props: CommandProps): LoeschAnforderungState => {
      return isVorgangLoeschenCommand(props.command.order) ?
          { ...state, vorgangLoeschenCommand: createStateResource(props.command) }
        : state;
    },
  ),

  //loeschAnforderungZuruecknehmenCommand
  on(
    CommandActions.createCommand,
    (state: LoeschAnforderungState, props: CreateCommandProps): LoeschAnforderungState => {
      return isLoeschAnforderungZuruecknehmen(props.command.order) ?
          {
            ...state,
            loeschAnforderungZuruecknehmenCommand: createEmptyStateResource(true),
          }
        : state;
    },
  ),
  on(
    CommandActions.createCommandSuccess,
    (state: LoeschAnforderungState, props: CommandProps): LoeschAnforderungState => {
      return isLoeschAnforderungZuruecknehmen(props.command.order) ?
          {
            ...state,
            loeschAnforderungZuruecknehmenCommand: createStateResource(props.command),
            loeschAnforderung: createEmptyStateResource(),
          }
        : state;
    },
  ),
  on(
    CommandActions.createCommandFailure,
    (state: LoeschAnforderungState, props: CreateCommandFailureProps): LoeschAnforderungState => {
      return isLoeschAnforderungZuruecknehmen(props.command.order) ?
          {
            ...state,
            loeschAnforderungZuruecknehmenCommand: createErrorStateResource(<ApiError>props.error),
          }
        : state;
    },
  ),
  on(
    CommandActions.revokeCommandSuccess,
    (state: LoeschAnforderungState, props: CommandProps): LoeschAnforderungState => {
      return isLoeschAnforderungZuruecknehmen(props.command.order) ?
          {
            ...state,
            loeschAnforderung: { ...createEmptyStateResource(), reload: true },
          }
        : state;
    },
  ),

  //VorgangActions
  on(
    VorgangActions.loadVorgangWithEingangSuccess,
    (state: LoeschAnforderungState): LoeschAnforderungState => {
      return {
        ...state,
        loeschenAnfordernCommand: createEmptyStateResource(),
        vorgangLoeschenCommand: createEmptyStateResource(),
        loeschAnforderung: createEmptyStateResource(),
        loeschAnforderungZuruecknehmenCommand: createEmptyStateResource(),
      };
    },
  ),

  //Navigation
  on(
    NavigationActions.updateCurrentRouteData,
    (state: LoeschAnforderungState): LoeschAnforderungState => {
      return {
        ...state,
        loeschenAnfordernCommand: createEmptyStateResource(),
        loeschAnforderung: createEmptyStateResource(),
        vorgangLoeschenCommand: createEmptyStateResource(),
        loeschAnforderungZuruecknehmenCommand: createEmptyStateResource(),
      };
    },
  ),
);

export function reducer(state: LoeschAnforderungState, action: Action): LoeschAnforderungState {
  return loeschAnforderungReducer(state, action);
}
